#ifndef __LOCALIZATION_USER_INTERFACE_H
#define __LOCALIZATION_USER_INTERFACE_H
//OpenCV Includes
#include "highgui.h"
#include <stdio.h>
#include <ctype.h>

//My includes
#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <string>
#include <time.h>

#include "global_vars.h"
#include "XBeePackets.h"
using namespace std;

//******************************************************//
//********This function captures the keystrokes**********//
//****To adjust the dynamic parameters of processing****//
//******************************************************//
int process_keys(void);
void command_console(void);

#define CHECK_BUFFER if (!cin.good()){\
						cout<<"error, clearing input buffer"<<endl;\
						cout<<"good:"<<cin.good()<<endl;\
						cout<<"eof :"<<cin.eof()<<endl;\
						cout<<"fail:"<<cin.fail()<<endl;\
						cout<<"bad :"<<cin.bad()<<endl;\
						cin.ignore(1000, '\n');\
						cin.clear();\
					}\
					else{\
						bytesSent = packetHandlerPtr->assemblePacket(packetData,targetID);\
					}

int process_keys(void)
{
	int c = cvWaitKey(10);
	if( (char) c == 27 )
	{
		// 'Esc' exits
		return (1);
	}
	if( (char) c == 13 )
	{
		// 'Return' used for setup (press enter to continue)
		return (2);
	}
	else if ( c == 'r')
	{
		// 'r' is used to restart all captures
		// this will assume the same ordering/phyiscal layout
		// of the cameras, but will disconnect the library and
		// attempt to reopen the source.

		// Used when a black image is seen from one of the cameras if the
		// capture source was not closed properly during the last run (commonly reason)
		for (int camerai = 0; camerai<4; camerai++)
		{
			VI->restartDevice(captureID[camerai]);
		}
	}
	else if ( c == 'e')
	{
		// e turns on and off erosion of the images.
		// default is not to use it, it doesn't seem particularly usefull with
		// the other patern and real time constraints
		erode = !erode;
		cout<<"Erosion is "<<erode<<endl;
	}
	else if ( (char) c == 43 || (char) c == 61)
	{
		// '+ or (shift+)'  Increases threshold for black/white
		if (threshold<255)
			threshold++;
		cout<<"Threshold: "<<threshold<<endl;
	}
	else if ( (char) c == 45)
	{
		// '-' Decreases threshold for black/white distinction
		if (threshold>0)
			threshold--;
		cout<<"Threshold: "<<threshold<<endl;
	}
	else if ( c == ']' )
	{
		// ']' Used to increase max distance between dots
		max_dot_distance = max_dot_distance + 1;
		cout<<"Max Dot Distance: "<<max_dot_distance<<endl;
	}
	else if ( c == '[' )
	{
		// '[' Used to decrease max distance between dots
		max_dot_distance = max_dot_distance - 1;
		cout<<"Max Dot Distance: "<<max_dot_distance<<endl;
	}
	else if ( c == 'x' )
	{
		// 'x' Used to incresae size of individual dot
		area_desired += 0.5;
		cout<<"area_desired: "<<area_desired<<endl;
	}
	else if ( c == 'z' )
	{
		// 'z' Used to decrease size of individual dot
		area_desired -= 0.5;
		cout<<"area_desired: "<<area_desired<<endl;
	}
	else if ( c == 'p'){
		while (cvWaitKey(10) != 'p'){};
	}
	else if ( c == 'c'){
		command_console();
	}
	else
	{
		if ((char) c != -1)
			cout<<"Keypress: "<<c<<endl;
	}
	return 0;
}

void command_console(void){
	char buffer1[100];
	UINT16 targetID;
	bool exitflag = 0;
	cout<<"Entered Command Console, 'exit' to return"<<endl;
	while(exitflag == 0){
		cout<<">>";
		cin>>buffer1;
		if (buffer1[0] == 'e' &&
			buffer1[1] == 'x' &&
			buffer1[2] == 'i' &&
			buffer1[3] == 't'){
			exitflag = 1;
			break;
		}
		else{
			cin>>targetID;
		}
		if (!cin.good()){
			cout<<"error, clearing input buffer"<<endl;
			cout<<"good:"<<cin.good()<<endl;
			cout<<"eof :"<<cin.eof()<<endl;
			cout<<"fail:"<<cin.fail()<<endl;
			cout<<"bad :"<<cin.bad()<<endl;
			cin.ignore(1000, '\n');
			cin.clear();
		}
		else{
			/*
			Command IDs:
			command		ID		Description      
			(n/a)		0.0		 coordinate data
			sleep		1.0		 sleep
			wake		2.0		 wake (resets x_i consensus values)
			goal		3.0		 change goal parameters
			goto		4.0		 change goal position only (formation is the same)
			deadb		5.0		 change	motor low-velocity deadband
			egain		6.0		 change estimator gains KP,KI,GAMMA
			cgain		7.0		 change controller gains
			commr		8.0		 change COMMR
			speed		9.0		 change MAX_WHEEL_V_TICKS
			safed		10.0	 change obstacle avoidance theshold SAFEDIST 
			*/

			int bytesSent;
			float packetData[12]={99.0};
			//sleep command
			if (buffer1[0] == 's' &&
				buffer1[1] == 'l' &&
				buffer1[2] == 'e' &&
				buffer1[3] == 'e' &&
				buffer1[4] == 'p' ){
					if (targetID == 0){
						targetID = 0xFFFF;
					}
					packetData[0] = 1.0;
					CHECK_BUFFER 
			}
			else if (buffer1[0] == 'w' &&
				buffer1[1] == 'a' &&
				buffer1[2] == 'k' &&
				buffer1[3] == 'e'){
				if (targetID == 0){
					targetID = 0xFFFF;
				}
				packetData[0] = 2.0;
				CHECK_BUFFER 
			}
			else if (buffer1[0] == 'g' &&
				buffer1[1] == 'o' &&
				buffer1[2] == 'a' &&
				buffer1[3] == 'l'){
					if (targetID == 0){
						targetID = 0xFFFF;
					}
					packetData[0] = 3.0;
					cin>>packetData[1]>>packetData[2]>>packetData[3]>>packetData[4]>>packetData[5];
					CHECK_BUFFER 
			}

			else if (buffer1[0] == 'g' &&
				buffer1[1] == 'o' &&
				buffer1[2] == 't' &&
				buffer1[3] == 'o'){
					if (targetID == 0){
						targetID = 0xFFFF;
					}
					packetData[0] = 4.0;
					cin>>packetData[1]>>packetData[2];
					CHECK_BUFFER 
			}

			else if (buffer1[0] == 'd' && 
				buffer1[1] == 'e' &&
				buffer1[2] == 'a' &&
				buffer1[3] == 'd' &&
				buffer1[4] == 'b'){
				if (targetID == 0){
					targetID = 0xFFFF;
				}
				packetData[0] = 5.0;
				cin>>packetData[1];
				CHECK_BUFFER 
			}

			else if (buffer1[0] == 'e' && //change estimator gains 
				buffer1[1] == 'g' &&
				buffer1[2] == 'a' &&
				buffer1[3] == 'i' &&
				buffer1[4] == 'n'){
				if (targetID == 0){
					targetID = 0xFFFF;
				}
				packetData[0] = 6.0;
				cin>>packetData[1]>>packetData[2]>>packetData[3];
				CHECK_BUFFER 
			}

			else if (buffer1[0] == 'c' && //change controller gains 
				buffer1[1] == 'g' &&
				buffer1[2] == 'a' &&
				buffer1[3] == 'i' &&
				buffer1[4] == 'n'){
				if (targetID == 0){
					targetID = 0xFFFF;
				}
				packetData[0] = 7.0;
				cin>>packetData[1]>>packetData[2]>>packetData[3]>>packetData[4]>>packetData[5];
				CHECK_BUFFER 
			}

			else if (buffer1[0] == 'c' && 
				buffer1[1] == 'o' &&
				buffer1[2] == 'm' &&
				buffer1[3] == 'm' &&
				buffer1[4] == 'r'){
				if (targetID == 0){
					targetID = 0xFFFF;
				}
				packetData[0] = 8.0;
				cin>>packetData[1];
				CHECK_BUFFER 
			}
			else if (buffer1[0] == 's' && 
				buffer1[1] == 'p' &&
				buffer1[2] == 'e' &&
				buffer1[3] == 'e' &&
				buffer1[4] == 'd'){
				if (targetID == 0){
					targetID = 0xFFFF;
				}
				packetData[0] = 9.0;
				cin>>packetData[1];
				CHECK_BUFFER 
			}

			else if (buffer1[0] == 's' &&
				buffer1[1] == 'a' &&
				buffer1[2] == 'f' &&
				buffer1[3] == 'e' &&
				buffer1[4] == 'd'){
					if (targetID == 0){
						targetID = 0xFFFF;
					}
					packetData[0] = 10.0;
					cin>>packetData[1];
					CHECK_BUFFER 
			}


			else{
				cout<<"*** Command not recognized ***"<<endl;
			}
			Sleep(100);
		}
	}
	return;
}

//******************************************************//
//********This captures mouse input from the GUI********//
//******************************************************//

void on_mouse( int event, int x, int y, int flags, void* param )
{
    if( !image )
        return;

    if( image->origin )
	{
		y = image->height - y;
	}
	
	// Left click event on GUI is used to orient cameras in 
	// from physical locations to USB PORTs on the computer
	if( event == CV_EVENT_LBUTTONDOWN )
	{
		if (num_cams_ordered < 4)	// user clicking to specify camera locations
		{
			if (x>guiWidth / 2)
			{
				if (y>guiHeight / 2)
				{
					cameraID[num_cams_ordered] = 3;
					cout<<"\nQ4\n";
				}
				else
				{
					cameraID[num_cams_ordered] = 0;
					cout<<"\nQ1\n";
				}
			}
			else
			{
				if (y>guiHeight / 2)
				{
					cameraID[num_cams_ordered] = 2;
					cout<<"\nQ3\n";
				}
				else
				{
					cameraID[num_cams_ordered] = 1;
					cout<<"\nQ2\n";
				}
			}
			num_cams_ordered++;
		}
		else if (camera_quadrants_matched == 0)
		{
			camera_quadrants_matched = 1;
		}
		else if (runMode == 0)
		{
			runMode = 1;
		}
	}

	// Right mouse click on GUI used to display a world location
	// with calibrated world coordinates
	if( ( event == CV_EVENT_RBUTTONDOWN )&&(calibration_complete == 1))
	{
		if (fWidth != thumbWidth || fHeight != thumbHeight){
			cout<<"Warning, images resized, coords will be off"<<endl;
		}
		if (x>guiWidth / 2)
		{
			if (y>guiHeight / 2)
			{
				cout<<"\nQ4\n";
				x = x-thumbWidth;
				y = y-thumbHeight;
				cvmSet(imageC,0,0,x);
				cvmSet(imageC,1,0,y);
				cvmSet(imageC,2,0,0);
				cvmSet(imageC,3,0,1);

				cvMatMul(camCalib[3],imageC, worldC);
				double worldX = cvmGet(worldC, 0,0)/cvmGet(worldC, 2,0);
				double worldY = cvmGet(worldC, 1,0)/cvmGet(worldC, 2,0);
				double worldZ = 1;

				cout<<" X:"<<x<<" Y: "<<y<<" \tXw: "<<worldX<<" Yw: "<<worldY<<endl;
			}
			else
			{
				cout<<"\nQ1\n";
				x = x-thumbWidth;
				cvmSet(imageC,0,0,x);
				cvmSet(imageC,1,0,y);
				cvmSet(imageC,2,0,0);
				cvmSet(imageC,3,0,1);

				cvMatMul(camCalib[0],imageC, worldC);
				double worldX = cvmGet(worldC, 0,0)/cvmGet(worldC, 2,0);
				double worldY = cvmGet(worldC, 1,0)/cvmGet(worldC, 2,0);
				double worldZ = 1;


				cout<<" X:"<<x<<" Y: "<<y<<" \tXw: "<<worldX<<" Yw: "<<worldY<<endl;
			}
		}
		else
		{
			if (y>guiHeight / 2)
			{
				cout<<"\nQ3\n";
				y = y-thumbHeight;
				cvmSet(imageC,0,0,x);
				cvmSet(imageC,1,0,y);
				cvmSet(imageC,2,0,0);
				cvmSet(imageC,3,0,1);

				cvMatMul(camCalib[2],imageC, worldC);
				double worldX = cvmGet(worldC, 0,0)/cvmGet(worldC, 2,0);
				double worldY = cvmGet(worldC, 1,0)/cvmGet(worldC, 2,0);
				double worldZ = 1;


				cout<<" X:"<<x<<" Y: "<<y<<" \tXw: "<<worldX<<" Yw: "<<worldY<<endl;
			}
			else
			{
				cout<<"\nQ2\n";
				cvmSet(imageC,0,0,x);
				cvmSet(imageC,1,0,y);
				cvmSet(imageC,2,0,0);
				cvmSet(imageC,3,0,1);

				cvMatMul(camCalib[1],imageC, worldC);
				double worldX = cvmGet(worldC, 0,0)/cvmGet(worldC, 2,0);
				double worldY = cvmGet(worldC, 1,0)/cvmGet(worldC, 2,0);
				double worldZ = 1;


				cout<<" X:"<<x<<" Y: "<<y<<" \tXw: "<<worldX<<" Yw: "<<worldY<<endl;
			}
		}
	}
}



//***************************************************
//***Ask to recalibrate cameras or not.
//***************************************************
bool AskRecalibrateCams(void){
	bool valid_input = 0;
	while (!valid_input){
		cout<<endl<<"Recalibrate cameras? y/n: ";
		char temp_buf;
		cin>>temp_buf;
		if (temp_buf == 'y'){
			valid_input = 1;
			return 1;
		}
		else if (temp_buf == 'n'){
			valid_input = 1;
			return 0;
		}
		else{
			cout<<"Invalid Input, try again"<<endl;
		}
	}
}



//******************************************************//
//********This function draws the main GUI**************//
//******************************************************//
int draw_gui(int mode = 1)
{	
	//set gui to tan
	cvSet(gui, CV_RGB(245,245,220), 0);
	
	// dynamically set up size of thumbnail
	IplImage *thumbnail = 0;
	thumbnail = cvCreateImage(cvSize(thumbWidth, thumbHeight), 8, 3 );
	
	if (mode == 1)			//Raw Images from cams (color)
	{
		//camera 1 (quadrant 1)
		cvResize(rawImage[0], thumbnail);
		cvSetImageROI(gui, cvRect(thumbWidth, 0, thumbWidth, thumbHeight));
		cvCopy(thumbnail, gui);
		//camera 2 (quadrant 2)
		cvResize(rawImage[1], thumbnail);
		cvSetImageROI(gui, cvRect((int) 0, 0, thumbWidth, thumbHeight));
		cvCopy(thumbnail, gui);
		//camera 3 (quadrant 3)
		cvResize(rawImage[2], thumbnail);
		cvSetImageROI(gui, cvRect((int)0, thumbHeight, thumbWidth, thumbHeight));
		cvCopy(thumbnail, gui);
		//camera 4 (quadrant 4)
		cvResize(rawImage[3], thumbnail);
		cvSetImageROI(gui, cvRect(thumbWidth, thumbHeight, thumbWidth, thumbHeight));
		cvCopy(thumbnail, gui);
	}
	else if (mode == 0)		//Thresholded B/W Images
	{
		//camera 1 (quadrant 1)
		cvCvtColor(thresholdedImage[0], thresholdedImageColor[0], CV_GRAY2BGR);
		cvResize(thresholdedImageColor[0], thumbnail);
		cvSetImageROI(gui, cvRect(thumbWidth, 0, thumbWidth, thumbHeight));
		cvCopy(thumbnail, gui);

		//camera 2 (quadrant 2)
		cvCvtColor(thresholdedImage[1], thresholdedImageColor[1], CV_GRAY2BGR);
		cvResize(thresholdedImageColor[1], thumbnail);
		cvSetImageROI(gui, cvRect(0, 0, thumbWidth, thumbHeight));
		cvCopy(thumbnail, gui);

		//camera 3 (quadrant 3)
		cvCvtColor(thresholdedImage[2], thresholdedImageColor[2], CV_GRAY2BGR);
		cvResize(thresholdedImageColor[2], thumbnail);
		cvSetImageROI(gui, cvRect(0, thumbHeight, thumbWidth, thumbHeight));
		cvCopy(thumbnail, gui);

		//camera 4 (quadrant 4)
		cvCvtColor(thresholdedImage[3], thresholdedImageColor[3], CV_GRAY2BGR);
		cvResize(thresholdedImageColor[3], thumbnail);
		cvSetImageROI(gui, cvRect(thumbWidth, thumbHeight, thumbWidth, thumbHeight));
		cvCopy(thumbnail, gui);
	}

	// Put Title Text
	cvResetImageROI(gui);
	CvFont font;
	cvInitFont(&font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, 0.75,0.75,0,2);
	string title_string = "Machine Vision Localization System";
	cvPutText (gui,title_string.c_str(),cvPoint((int) guiWidth/8,(int) 20), &font, CV_RGB(0,0,0));
	cvInitFont(&font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, 0.5,0.5,0,1);
	title_string = "Version RGB 8/2009";
	cvPutText (gui,title_string.c_str(),cvPoint((int) guiWidth/3.5,(int) 45), &font, CV_RGB(0,0,0));
	title_string = "Scott McLeod - Northwestern University - modified by M. Hwang and J. Lee";
	cvPutText (gui,title_string.c_str(),cvPoint((int) guiWidth/8,(int) 65), &font, CV_RGB(0,0,0));

	// Put Message Text
	cvPutText (gui,message1.c_str(),cvPoint((int) 10,(int) guiHeight - 50), &font, CV_RGB(0,0,255));
	cvPutText (gui,message2.c_str(),cvPoint((int) 10,(int) guiHeight - 30), &font, CV_RGB(0,0,255));
	cvReleaseImage(&thumbnail);
	return 0;
}


void convertCvtColor(void){
	cvCvtColor(thresholdedImage[0], thresholdedImageColor[0], CV_GRAY2BGR);
	cvCvtColor(thresholdedImage[1], thresholdedImageColor[1], CV_GRAY2BGR);
	cvCvtColor(thresholdedImage[2], thresholdedImageColor[2], CV_GRAY2BGR);
	cvCvtColor(thresholdedImage[3], thresholdedImageColor[3], CV_GRAY2BGR);
}

int draw_gui_NoCvtColor(void)
{	
	//set gui to tan
	cvSet(gui, CV_RGB(245,245,220), 0);
	
	// dynamically set up size of thumbnail
	IplImage *thumbnail = 0;
	thumbnail = cvCreateImage(cvSize(thumbWidth, thumbHeight), 8, 3 );

	//camera 1 (quadrant 1)
	cvResize(thresholdedImageColor[0], thumbnail);
	cvSetImageROI(gui, cvRect(thumbWidth, 0, thumbWidth, thumbHeight));
	cvCopy(thumbnail, gui);

	//camera 2 (quadrant 2)
	cvResize(thresholdedImageColor[1], thumbnail);
	cvSetImageROI(gui, cvRect(0, 0, thumbWidth, thumbHeight));
	cvCopy(thumbnail, gui);

	//camera 3 (quadrant 3)
	cvResize(thresholdedImageColor[2], thumbnail);
	cvSetImageROI(gui, cvRect(0, thumbHeight, thumbWidth, thumbHeight));
	cvCopy(thumbnail, gui);

	//camera 4 (quadrant 4)
	cvResize(thresholdedImageColor[3], thumbnail);
	cvSetImageROI(gui, cvRect(thumbWidth, thumbHeight, thumbWidth, thumbHeight));
	cvCopy(thumbnail, gui);


	// Put Title Text
	cvResetImageROI(gui);
	CvFont font;
	cvInitFont(&font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, 0.75,0.75,0,2);
	string title_string = "Machine Vision Localization System";
	cvPutText (gui,title_string.c_str(),cvPoint((int) guiWidth/8,(int) 20), &font, CV_RGB(0,0,0));
	cvInitFont(&font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, 0.5,0.5,0,1);
	title_string = "Version RGB 8/2009";
	cvPutText (gui,title_string.c_str(),cvPoint((int) guiWidth/3.5,(int) 45), &font, CV_RGB(0,0,0));
	title_string = "Scott McLeod - Northwestern University - modified by M. Hwang and J. Lee";
	cvPutText (gui,title_string.c_str(),cvPoint((int) guiWidth/8,(int) 65), &font, CV_RGB(0,0,0));

	// Put Message Text
	cvPutText (gui,message1.c_str(),cvPoint((int) 10,(int) guiHeight - 50), &font, CV_RGB(0,0,255));
	cvPutText (gui,message2.c_str(),cvPoint((int) 10,(int) guiHeight - 30), &font, CV_RGB(0,0,255));
	cvReleaseImage(&thumbnail);
	return 0;
}


double getFPS(void){
		long newtime;
		static long lastFrameT[3]={-1,-1,-1};
		double fps;
		newtime = clock();
		fps = 1/((newtime - lastFrameT[0])/1000.0);
		lastFrameT[2]=lastFrameT[1];
		lastFrameT[1]=lastFrameT[0];
		lastFrameT[0]=newtime;
		return fps;
}
#endif 