//Please refer to the Machine Vision Localization System for more information about the theory and the exact functions of this vision system
//One can find this on the LIMS/Mechatronics wiki under that name (Machine Vision Localization System)
//This version of the code was modified to be used with the RGB Swarm Robot Project
//
//jonathan lee 08/2009

//#define SPOT_TEST  //defining SPOT_TEST will make the program just print out the location of spots.
//#define DEBUG_MODE //defining DEBUG_MODE will print out position data

//******************************************************//
//********INCLUDES**************************************//
//******************************************************//
#include "stdafx.h"
#include <crtdbg.h>

//OpenCV Includes
#include "cv.h"
#include "highgui.h"
#include <stdio.h>
#include <ctype.h>

//VideoInput Library
#include "videoInput.h"


//My includes
#include <iostream>
#include <iomanip>
#include <sstream>
#include <string>
#include <math.h>
#include <time.h>
//#include <windows.h>
//#include "Serial.h"
#include "CameraCalibration.h"
#include "global_vars.h"
#include "LocalizationFileIO.h"
#include "LocalizationUserInterface.h"


#include "serialport.h"
#include "XBeePackets.h"
#include "utilities.h"

using namespace std;


//******************************************************//
//***This function converts to grey and binary**********//
//******************************************************//
void threshold_images()
{
	for(int camerai = 0; camerai<4; camerai++)
	{
		cvCvtColor( rawImage[camerai], greyImage[camerai], CV_BGR2GRAY );
		cvThreshold(greyImage[camerai], thresholdedImage[camerai], threshold, 255, CV_THRESH_BINARY);
	}
	return;
}

//******************************************************//
//*****This function gets raw frames from Cams**********//
//******************************************************//
void get_frames()
{
	// Get frames of cameras and put into appropriate images
	for(int camerai = 0; camerai<4; camerai++)
	{
		if (cameraID[camerai] != -1)  //make sure camera is initialized
		{
			if(VI->isFrameNew(captureID[camerai]))
			{
				new_image = VI->getPixels(captureID[camerai], false, true);
				rawImage[cameraID[camerai]]->imageData = (char*) new_image;  
			}
		}
	}
	return;
}

//******************************************************//
//****************The main program loop*****************//
//******************************************************//
int main( int argc, char** argv )
{
	CvFont gui_font;
	cvInitFont(&gui_font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, 1,1,0,2);
	CvFont gui_font_2;
	cvInitFont(&gui_font_2,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, 0.5,0.5,0,1);

	classifier* first_classifier;
	first_classifier = LoadTargetData();

	classifier* current_classifier;
	current_classifier = first_classifier;
/*
	cout<<"Scott McLeod Indoor Localization System v1.0\nMarch 2008\n";
	cout<<"Serial Port Setup.\n";
	cout<<"Type the name of the serial port (e.g. 'COM1')\n";
	string serialName;
	//cin>>serialName;
serialName = "COM1";

	CSerial serialout;
	cout<<"Attempting to open the serial port.\n";
	serialout.Open(serialName.c_str());
	cout<<"Testing "<<serialName<<"...\n";
	cout<<"Type 't' to test connection.  Type 'q' to stop testing"<<endl;
	char sentinel = 't';
/*
	do
	{	
		string test = "Hello World on Port ";
		test.append(serialName);
		test.append("\n");
		serialout.Write(test.c_str());
		cin>>sentinel;
	} while (sentinel == 't');
*/

//start added serial
//create serial port object
	CSerialPort COM;	
	COMMTIMEOUTS timeouts;
	DWORD dwErrors;
	PacketHandler packetHandler(COM);
	packetHandlerPtr = &packetHandler;
	XBeeInPacket inPacket;
	int COMNum;
	cout<<"Enter COM number:";
	cin>>COMNum;

	try {
//Open COM Port (XBee Radio doesn't like 115200 baud, this was 115000)
		COM.Open(COMNum, 115000, CSerialPort::NoParity, 8, 
		CSerialPort::OneStopBit, 
		CSerialPort::NoFlowControl);

///Timeouts///			
		timeouts.ReadIntervalTimeout = -1; //-1 is no waiting, 0 is infinite waiting
		timeouts.ReadTotalTimeoutMultiplier = 0;
		timeouts.ReadTotalTimeoutConstant = 0;
		timeouts.WriteTotalTimeoutMultiplier = 0;
		timeouts.WriteTotalTimeoutConstant = 0;
		COM.SetTimeouts(timeouts);
///Clear COM Port
		COM.Purge(PURGE_RXCLEAR);
		COM.Flush();
		COM.ClearError(dwErrors);
///
		///main loop

	}
	catch (CSerialException* pEx)
	{
		TRACE(_T("Handle Exception, Message:%s\n"), 
		pEx->GetErrorMessage());
		pEx->Delete();
		cout<<"failure opening serial port"<<endl;
		cin.get();
	}



//end added serial

	cout<<"\n\n\nOpening Video Capture Devices\n\n";
		
	VI = new videoInput();
	VI->listDevices();
	
	for(int camerai = 0; camerai <4; camerai++)
	{
		cout<<"Select Capture Device "<<camerai+1<<": ";
		//cin>>captureID[camerai];
captureID[camerai]=camerai;
	}

	
	// connect to all 4 cameras
	for (int camerai = 0; camerai<4; camerai++)
	{
		VI->setupDevice(captureID[camerai],fWidth, fHeight);
	}

	// create CV windows
	cvNamedWindow( mainWindow.c_str(), 1 );
	cvNamedWindow( vidWindow.c_str(), 1 );

	// set the mouse callback
    cvSetMouseCallback( mainWindow.c_str(), on_mouse, 0 );

    
	//initialize CCL storage elements
	CvMemStorage* storage = cvCreateMemStorage(0);
    CvSeq* contour = 0;

	//initialize file writing 
	fOut.open("output.txt");
	//fX.open("X.txt");
	//fY.open("Y.txt");
	//fA.open("Angle.txt");
	
	//intialize video writers
	// These were used for saving results to video files for intermediate
	// Documentation and presentation.
	//CvVideoWriter *writer1 = 0;
	//CvVideoWriter *writer2 = 0;
	//writer1 = cvCreateVideoWriter("Contoured.avi", CV_FOURCC('D', 'I', 'V', 'X'), 29, cvSize(fWidth, fHeight), 1);
	//writer2 = cvCreateVideoWriter("Original.avi", CV_FOURCC('D', 'I', 'V', 'X'), 29, cvSize(fWidth, fHeight), 1);
	
	//initialize images
	image = cvCreateImage(cvSize(fWidth, fHeight), 8, 3 );
	grey = cvCreateImage(cvSize(fWidth, fHeight), 8, 1 );
	thresholded_image = cvCreateImage(cvSize(fWidth, fHeight), 8, 1 );
	circle_image = cvCreateImage(cvSize(fWidth, fHeight), 8, 1 );
	world_image = cvCreateImage( cvSize(fWidth, fHeight), 8, 3 );
	gui = cvCreateImage(cvSize(guiWidth, guiHeight), 8, 3);

	// Init openCV drawing FONT
	CvFont font;
	cvInitFont(&font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, 3,3,0,2);
	
	// Draw 1 2 3 4 for quadrants
	string title_string = "1";
	rawImage[0] = cvCreateImage(cvSize(fWidth, fHeight), 8, 3 );
	cvPutText (rawImage[0],title_string.c_str(),cvPoint((int) fWidth/2-20,(int) fHeight/2-20), &font, CV_RGB(0,255,255));
	title_string = "2";
	rawImage[1] = cvCreateImage(cvSize(fWidth, fHeight), 8, 3 );
	cvPutText (rawImage[1],title_string.c_str(),cvPoint((int) fWidth/2-20,(int) fHeight/2-20), &font, CV_RGB(0,255,255));
	title_string = "3";
	rawImage[2] = cvCreateImage(cvSize(fWidth, fHeight), 8, 3 );
	cvPutText (rawImage[2],title_string.c_str(),cvPoint((int) fWidth/2-20,(int) fHeight/2-20), &font, CV_RGB(0,255,255));
	title_string = "4";
	rawImage[3] = cvCreateImage(cvSize(fWidth, fHeight), 8, 3 );
	cvPutText (rawImage[3],title_string.c_str(),cvPoint((int) fWidth/2-20,(int) fHeight/2-20), &font, CV_RGB(0,255,255));

	for(int camerai = 0; camerai<4; camerai++)
	{
		greyImage[camerai] = cvCreateImage(cvSize(fWidth, fHeight), 8, 1 );
		thresholdedImage[camerai] = cvCreateImage(cvSize(fWidth, fHeight), 8, 1 );
		thresholdedImageColor[camerai] = cvCreateImage(cvSize(fWidth, fHeight), 8, 3 );
	}

	//******************************************************//
	//*****PHASE 1: CAMERA SETUP/ORDERING/POSITING LOOP*****//
	//******************************************************//
	// This is when the user sees a window with a cpature source
	// They must pair the capture source with the location in the GUI/world
	message1 = "Click Quadrant Corresponding to Camera";
	message2 = "Press 'r' to restart cameras (if no image visible)";
	while(num_cams_ordered < 4)
	{
		// Update vidWindow (window 2)
		new_image = VI->getPixels(captureID[num_cams_ordered], false, true);
		image->imageData = (char*) new_image;
		cvShowImage(vidWindow.c_str(), image);
		
		// Update main GUI
		get_frames();
		draw_gui();
		cvShowImage( mainWindow.c_str(), gui);
		
		// Allow Keystrokes
		if(process_keys()==1)
		{
			break;
		}
	}
	cvDestroyWindow(vidWindow.c_str());

	message1 = "Check to Make Sure Accurate Alignment";
	message2 = "Press 'Enter' to Continue";
	while(1)
	{
		get_frames();
		draw_gui();
		cvShowImage(mainWindow.c_str(), gui);
		// Allow Keystrokes
		if(process_keys()==2)
		{
			break;
		}
		
	}

	//******************************************************//
	//********PHASE 2a: WORLD CALIBRATION LOOP**************//
	//******************************************************//
	//Show template image, and input arguments for sizings
/*
	IplImage *template_image = cvLoadImage("world_coord_template.jpg");

	double idd_hor, idd_ver;

	message1 = "Enter Template Pattern Constraints";
	message2 = "Type Parameter A in command prompt.";
	draw_gui(1);
	cvSetImageROI(gui, cvRect(0, guiHeight/6, template_image->width, template_image->height));
	cvCopy(template_image, gui);
	cvResetImageROI(gui);
	cvShowImage(mainWindow.c_str(), gui);
	cvWaitKey(100);
	cout<<"\nEnter parameter A in unitless double format: ";
//	cin>>idd_dist;
	
	message2 = "Type Parameter B in command prompt.";
	draw_gui(1);
	cvSetImageROI(gui, cvRect(0, guiHeight/6, template_image->width, template_image->height));
	cvCopy(template_image, gui);
	cvResetImageROI(gui);
	cvShowImage(mainWindow.c_str(), gui);
	cvWaitKey(100);
	cout<<"\nEnter parameter B in unitless double format: ";
//	cin>>itd_ver;
	
	message2 = "Type Parameter C in command prompt.";
	draw_gui(1);
	cvSetImageROI(gui, cvRect(0, guiHeight/6, template_image->width, template_image->height));
	cvCopy(template_image, gui);
	cvResetImageROI(gui);
	cvShowImage(mainWindow.c_str(), gui);
	cvWaitKey(100);
	cout<<"\nEnter parameter C in unitless double format: ";
//	cin>>itd_hor;
*/

	recalibrate_cameras = AskRecalibrateCams();
	double idd_hor, idd_ver;	//inter-dot distances for calibrations

	if (recalibrate_cameras == 1){
		cout<<"\nEnter horizontal spacing of calibration dots (e.g. 635.0) (use mm for swarm project): ";
		cin>>idd_hor;
		cout<<"\nEnter vertical spacing of calibration dots (e.g. 508.0) (use mm for swarm project): ";
		cin>>idd_ver;
		WriteDotSpacing(idd_hor, idd_ver);
	}
	else{
		ReadDotSpacing(&idd_hor, &idd_ver);
	}
	
//	idd_ver = 508;
//	idd_hor = 685.8;

	//gerenate the world coordinate matrices.
	// This computes the location of all 9 x 4 = 28 dots
	// Assuming they are placed in a perfectly rectangular pattern
	//set origin to lower left (Q3) bottom left "dot"

	cout<<endl<<endl;
	int index = 0;
	for(int col = 0; col<3; col+=1)
	{
		for(int row = 0; row<3; row+=1)
		{
			world_cam1[index] = idd_hor*(col);
			cout<<world_cam1[index]<<",";
			world_cam1[index+1] = idd_ver*2-idd_ver*(row);
			cout<<world_cam1[index+1]<<"\t";
			index+=2;
		}
		cout<<endl;
	}
	cout<<"Q1";
	for(int i = 0; i<18; i++)
	{
		cout<<"element: "<<i<<": "<<world_cam1[i]<<endl;
	}

	//define all other cameras based on idd_hor and idd_vert
	//simply offset the 3x3 grid in the distances defined 'B'/'C'
	for (int i = 0; i<18; i+=2)
	{
		world_cam2[i] = world_cam1[i] - 2*idd_hor;
		world_cam2[i+1] = world_cam1[i+1];

		world_cam3[i] = world_cam1[i] - 2*idd_hor;
		world_cam3[i+1] = world_cam1[i+1] - 2*idd_ver;

		world_cam4[i] = world_cam1[i];
		world_cam4[i+1] = world_cam1[i+1] - 2*idd_ver;
	}

	//******************************************************//
	//********PHASE 2b: CAMERA CALIBRATION LOOP**************//
	//******************************************************//
	message1 = "Align Calibration Pattern";
	message2 = "Press 'Enter' to Calibrate to World.";
	//message2 = "Use +/- to Adjust Threshold.";
	
	for(int camerai = 0; camerai<4; camerai++)
	{
		rawImage[camerai] = cvCreateImage(cvSize(fWidth, fHeight), 8, 3 );
		greyImage[camerai] = cvCreateImage(cvSize(fWidth, fHeight), 8, 1 );
		thresholdedImage[camerai] = cvCreateImage(cvSize(fWidth, fHeight), 8, 1 );
		thresholdedImageColor[camerai] = cvCreateImage(cvSize(fWidth, fHeight), 8, 3 );
	}

	cout<<"Number of dots seen (each camera should see 9 dots for calibration):"<<endl;
	cout<<setw(20)<<"Quadrant 1"<<setw(15)<<"Quadrant 2"<<setw(15)<<"Quadrant 3"<<setw(15)<<"Quadrant 4"<<endl;
	while(1)
	{
		// update GUI
		get_frames();
		// Threshold images
		threshold_images();
		draw_gui(0);
		cvShowImage( mainWindow.c_str(), gui);

		//display number of dots detected
		int num_dots_detected = 0;
		for(int quadrant = 0; quadrant<4; quadrant++)
		{
			num_dots_detected = 0;
			// process image, find countours and dots
			cvClearMemStorage(storage);
			cvFindContours(thresholdedImage[quadrant], storage, &contour, sizeof(CvContour), CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE );
			for(contour; contour; contour=contour->h_next)
			{
				num_dots_detected++;
			}
			cout<<setw(15)<<num_dots_detected;
		}
		cout<<"\r";

		// Allow Keystrokes (enter to continue)
		// When the user hits enter, the process_keys() loop will break
		// and the current frames will be captured for calibration
		if(process_keys()==2)
		{
			break;
		}
	}
	
	message1 = "Calculating Camera Parameters";
	message2 = "Please wait...";
	for(int quadrant_i = 0; quadrant_i<4; quadrant_i++)
	{
		//setup new target to hold dot locations
		target *calib_pattern = new target;
		calib_pattern->numDots = 0;

		if (recalibrate_cameras == 1){
			// process image, find countours and dots
			cvClearMemStorage(storage);
			cvFindContours(thresholdedImage[quadrant_i], storage, &contour, sizeof(CvContour), CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE );
			for(contour; contour; contour=contour->h_next)
			{
				//find moments of contours
				CvMoments moments;
				cvContourMoments(contour, &moments);
				double area = cvGetSpatialMoment(&moments, 0, 0);
				double centerx = cvGetSpatialMoment(&moments, 1, 0) / area;
				double centery = cvGetSpatialMoment(&moments, 0, 1) / area;
				if (calib_pattern->numDots < 9)
				{
					calib_pattern->dots[calib_pattern->numDots].ix = centerx;
					calib_pattern->dots[calib_pattern->numDots].iy = centery;
					calib_pattern->numDots++;
				}
			}
		}
		else {
		//read in calibration dot locations from file
			ReadDotLocations(calib_pattern, quadrant_i);
		}

		// sort dots
		target *calib_pattern_sorted = new target;
		for(int round = 0; round < 3; round++)
		{
			double minX1, minX2, minX3;
			int posX1, posX2, posX3;
			minX1 = minX2 = minX3 = 100000;
			posX1 = posX2 = posX3 = 0;
			// Sort X's
			for (int i = 0; i<calib_pattern->numDots; i++)
			{
				if(calib_pattern->dots[i].ix<minX3)
				{
					if (calib_pattern->dots[i].ix<minX2)
					{
						if (calib_pattern->dots[i].ix<minX1)
						{
							minX3 = minX2;
							posX3 = posX2;
							minX2 = minX1;
							posX2 = posX1;
							minX1 = calib_pattern->dots[i].ix;
							posX1 = i;
						}
						else
						{
							minX3 = minX2;
							posX3 = posX2;
							minX2 = calib_pattern->dots[i].ix;
							posX2 = i;
						}
					}
					else
					{
						minX3 = calib_pattern->dots[i].ix;
						posX3 = i;
					}
				}
			}
			//Sort Y's
			if (calib_pattern->dots[posX1].iy < calib_pattern->dots[posX2].iy)
			{
				if (calib_pattern->dots[posX1].iy < calib_pattern->dots[posX3].iy)
				{
					calib_pattern_sorted->dots[round*3 + 0] = calib_pattern->dots[posX1];
					if (calib_pattern->dots[posX2].iy < calib_pattern->dots[posX3].iy)
					{
						calib_pattern_sorted->dots[round*3 + 1] = calib_pattern->dots[posX2];
						calib_pattern_sorted->dots[round*3 + 2] = calib_pattern->dots[posX3];
					}
					else
					{
						calib_pattern_sorted->dots[round*3 + 1] = calib_pattern->dots[posX3];
						calib_pattern_sorted->dots[round*3 + 2] = calib_pattern->dots[posX2];
					}
				}
				else
				{
					calib_pattern_sorted->dots[round*3 + 0] = calib_pattern->dots[posX3];
					calib_pattern_sorted->dots[round*3 + 1] = calib_pattern->dots[posX1];
					calib_pattern_sorted->dots[round*3 + 2] = calib_pattern->dots[posX2];
				}	
			}
			else
			{
				if (calib_pattern->dots[posX2].iy < calib_pattern->dots[posX3].iy)
				{
					calib_pattern_sorted->dots[round*3 + 0] = calib_pattern->dots[posX2];
					if (calib_pattern->dots[posX1].iy < calib_pattern->dots[posX3].iy)
					{
						calib_pattern_sorted->dots[round*3 + 1] = calib_pattern->dots[posX1];
						calib_pattern_sorted->dots[round*3 + 2] = calib_pattern->dots[posX3];
					}
					else
					{
						calib_pattern_sorted->dots[round*3 + 1] = calib_pattern->dots[posX3];
						calib_pattern_sorted->dots[round*3 + 2] = calib_pattern->dots[posX1];
					}
				}
				else
				{
					calib_pattern_sorted->dots[round*3 + 0] = calib_pattern->dots[posX3];
					calib_pattern_sorted->dots[round*3 + 1] = calib_pattern->dots[posX2];
					calib_pattern_sorted->dots[round*3 + 2] = calib_pattern->dots[posX1];
				}	
			}

			calib_pattern->dots[posX1].ix = 100000;
			calib_pattern->dots[posX2].ix = 100000;
			calib_pattern->dots[posX3].ix = 100000;
		}

		//Generate calibration matrices
		double image_coords[9][3];
		for(int i = 0; i<9; i++)
		{
			image_coords[i][0] = calib_pattern_sorted->dots[i].ix;
			image_coords[i][1] = calib_pattern_sorted->dots[i].iy;
			image_coords[i][2] = 1;
		}
		//write dot coordinates to file
		if (recalibrate_cameras == 1){
			WriteDotLocations(calib_pattern_sorted, quadrant_i);
		}

		// Call camera calibration function to get the 4 calibration matricies
		if (quadrant_i == 0)
		{
			camCalib[quadrant_i] = generate_calib_matrix_advanced_9(world_cam1, image_coords);
		}
		else if (quadrant_i == 1)
		{
			camCalib[quadrant_i] = generate_calib_matrix_advanced_9(world_cam2, image_coords);
		}
		else if (quadrant_i == 2)
		{
			camCalib[quadrant_i] = generate_calib_matrix_advanced_9(world_cam3, image_coords);
		}
		else if (quadrant_i == 3)
		{
			camCalib[quadrant_i] = generate_calib_matrix_advanced_9(world_cam4, image_coords);
		}
	}

	calibration_complete = 1;
	cout<<"phase3"<<endl;
	cin.get();
	//******************************************************//
	//********PHASE 3a: PREP MAIN PROGRAM LOOP**************//
	//******************************************************//	
	message1 = "Remove the Calibration Patterns";
	message2 = "Press 'Enter' when ready to start";
	while(1)
	{
		// update GUI
		get_frames();
		// Threshold images
		threshold_images();
		draw_gui(0);  
		cvShowImage( mainWindow.c_str(), gui);

		// Allow Keystrokes (Enter exits)
		if(process_keys()==2)
		{
			break;
		}
	}
#if defined SPOT_TEST
//added
	while(1)
	{
		process_keys();
		// update GUI
		get_frames();
		// Threshold images
		threshold_images();
		draw_gui(0);
		cvShowImage( mainWindow.c_str(), gui);
		for(int camerai = 0; camerai<4; camerai++)
		{
			// process image, find countours and dots
			cvClearMemStorage(storage);
			cvFindContours(thresholdedImage[cameraID[camerai]], storage, &contour, sizeof(CvContour), CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE );
			for(contour; contour; contour=contour->h_next)
			{
				//find moments of contours
				CvMoments moments;
				cvContourMoments(contour, &moments);
				double area = cvGetSpatialMoment(&moments, 0, 0);
				// area thresholding
						if (area>area_desired)
						{
							double centerx = cvGetSpatialMoment(&moments, 1, 0) / area;
							double centery = cvGetSpatialMoment(&moments, 0, 1) / area;

							//Convert Image Coordinates to World Coordinates
							cvSetZero(imageC);
						
							//Set up image Matrix
							cvmSet(imageC,0,0,centerx);
							cvmSet(imageC,1,0,centery);
							cvmSet(imageC,2,0,1);
							cvmSet(imageC,3,0,1);

							//Multiply by Camera Calibration Matrix
							cvMatMul(camCalib[cameraID[camerai]],imageC, worldC);

							//Extract world coordinates
							
							double worldX = cvmGet(worldC, 0,0)/cvmGet(worldC, 2,0);
							double worldY = cvmGet(worldC, 1,0)/cvmGet(worldC, 2,0);
							double worldZ = 1;

					cout<<worldX<<"\t"<<worldY<<endl;
					
				}
			}
		}

// Allow Keystrokes (enter to continue)
		// When the user hits enter, the process_keys() loop will break
		// and the current frames will be captured for calibration
		if(process_keys()==2)
		{
			break;
		}
	}
//end added
#endif
		
	//******************************************************//
	//********PHASE 3b: MAIN PROGRAM LOOP*******************//
	//******************************************************//	
	message1 = "Real-Time Localization Active. Press 'p' to pause, 'c' for command console, and 'Esc' to Quit";
	message2 = "";
	startTime = clock();
	int camScan[4]= {-1, -1, -1, -1};
	int send_counter = 0;
	while(1)
	{
		get_frames();
		threshold_images();
		convertCvtColor();
		//draw_gui(0);
		cvShowImage( mainWindow.c_str(), gui);

		//setup temp pointer to target list
		target* targets_temp[4];

		// setup scanning variables
		for(int i = 0; i<4; i++){
			camScan[i]=-1;
		}
		int numScanned = 0;
		int camerai = 0;


		// scan all 4 cameras before processing
		while (numScanned != 4)
		{
			camerai++;
			if (camerai > 3)
			{
				camerai = 0;
			}
			// camera setup and camera's frame hasnt' been grabbed
			// if camScan == -1, then the camera hasn't been checked
			// This ensures that all 4 cameras are processed before the
			// Same camera is processed twice.  VI checks if the frames are new
			// to prevent redundant checking of the same frame.
			if (cameraID[camerai] != -1 && camScan[camerai] == -1 )
			{
				if(VI->isFrameNew(captureID[camerai]))
				{
					//mark it gave frame and increase count
					camScan[camerai] = 0;
					numScanned++;
					
					//capture image
					new_image = VI->getPixels(captureID[camerai], false, true);
					image->imageData = (char*) new_image;
					//convert to b/w and threshold
					cvCvtColor( image, grey, CV_BGR2GRAY );
					cvThreshold(grey, thresholded_image, threshold, 255, CV_THRESH_BINARY);
					//find countours CCL (locate dots)
					cvClearMemStorage(storage);
					cvFindContours(thresholded_image, storage, &contour, sizeof(CvContour), CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE );
					CvScalar color1, color2;
					color1 = CV_RGB(255, 0, 0);
					color2 = CV_RGB(0, 255, 0);

					// set up moments

					double totalx = 0, totaly = 0;
					double centerCX = 0, centerCY = 0;
					
					//clear storage
					target* first_target = new target;
					first_target->numDots = 0;
					first_target->nxt = NULL;

					//for all contours, find dots and add them to storage
					CvMoments moments;
					int i = 0;
//added
					int num_contours=0;
					int num_accepted_dots=0;
//end added
					for(contour;contour; contour=contour->h_next)
					{
						num_contours++;
						i++;	
						//find moments of contours
						cvContourMoments(contour, &moments);
						double area = cvGetSpatialMoment(&moments, 0, 0);
//cout<<area<<endl;
						
						// area thresholding
						if (area>area_desired)
						{
							double centerx = cvGetSpatialMoment(&moments, 1, 0) / area;//this is the local center of mass, x coordinate
							double centery = cvGetSpatialMoment(&moments, 0, 1) / area;//this is the local center of mass, y coordinate
							num_accepted_dots++;
							
							//Convert Image Coordinates to World Coordinates
							cvSetZero(imageC);
							
							//Set up image Matrix
							cvmSet(imageC,0,0,centerx);
							cvmSet(imageC,1,0,centery);
							cvmSet(imageC,2,0,1);
							cvmSet(imageC,3,0,1);

							//Multiply by Camera Calibration Matrix
							cvMatMul(camCalib[cameraID[camerai]],imageC, worldC);

							//Extract world coordinates
							
							double worldX = cvmGet(worldC, 0,0)/cvmGet(worldC, 2,0);
							double worldY = cvmGet(worldC, 1,0)/cvmGet(worldC, 2,0);
							double worldZ = 1;
							


//cout<<"coords: \t"<< worldX<<setw(10)<<worldY<<setw(10)<<worldZ<<endl;

							//check for best fit target, create new otherwise
							target* temp = first_target;
							bool found_match = false;

							if ((first_target->nxt == NULL) && (first_target->numDots == 0))
							{
								first_target->dots[0].ix = centerx;
								first_target->dots[0].iy = centery;
								first_target->dots[0].wx = worldX;
								first_target->dots[0].wy = worldY;
								first_target->dots[0].wz = worldZ;
								first_target->dots[0].area = area;
								first_target->numDots = 1;
								found_match = true;
							}

							while((temp) && (!found_match))
							{
								//look for matching dot in each group
								for (i = 0; i<temp->numDots; i++)
								{
									double dist = sqrt(pow((worldX - temp->dots[i].wx),2) + pow( (worldY - temp->dots[i].wy),2));
									// If the distance between this dot and another inside a group is small enough
									// then associate this dot with this group "target"
									if ( (dist < max_dot_distance)  && (temp->numDots<9) && (!found_match))  
									{
										temp->dots[temp->numDots].ix = centerx;
										temp->dots[temp->numDots].iy = centery;
										temp->dots[temp->numDots].wx = worldX;
										temp->dots[temp->numDots].wy = worldY;
										temp->dots[temp->numDots].wz = worldZ;
										temp->dots[temp->numDots].area = area;
										temp->numDots++;
										found_match = true;
									}
								}
								temp = temp->nxt;
							}

							//didnt' find a match, make a new target object for the dot
							if (!found_match)
							{
								target* new_target = new target;
								new_target->numDots = 1;
								new_target->nxt = NULL;

								new_target->dots[0].ix = centerx;
								new_target->dots[0].iy = centery;
								new_target->dots[0].wx = worldX;
								new_target->dots[0].wy = worldY;
								new_target->dots[0].wz = worldZ;
								new_target->dots[0].area = area;
								//find last node
								
								temp = first_target;
								while(temp->nxt != NULL)
								{
									temp = temp->nxt;
								}
								temp->nxt = new_target;
							}
						}
					}

#if defined DEBUG_MODE
					cout<<"Camera: "<<camerai<<endl;
					cout<<"Num contours: "<<num_contours<<endl;
					cout<<"Num accepted dots: "<<num_accepted_dots<<endl<<endl;
#endif
					// All the dots have now been sorted into targets
					//find group center of mass
					targets_temp[camerai] = first_target;
					while(targets_temp[camerai])
					{
						targets_temp[camerai]->group.wx = 0;
						targets_temp[camerai]->group.wy = 0;
						targets_temp[camerai]->group.wz = 0;
						targets_temp[camerai]->group.ix = 0;
						targets_temp[camerai]->group.iy = 0;
						targets_temp[camerai]->group.area = 0;

						for(int i = 0; i<targets_temp[camerai]->numDots; i++)
						{
							targets_temp[camerai]->group.wx += targets_temp[camerai]->dots[i].wx;
							targets_temp[camerai]->group.wy += targets_temp[camerai]->dots[i].wy;
							targets_temp[camerai]->group.wz += targets_temp[camerai]->dots[i].wz;
							targets_temp[camerai]->group.ix += targets_temp[camerai]->dots[i].ix;
							targets_temp[camerai]->group.iy += targets_temp[camerai]->dots[i].iy;
							targets_temp[camerai]->group.area += targets_temp[camerai]->dots[i].area;
						}
						targets_temp[camerai]->group.ix /= targets_temp[camerai]->numDots;
						targets_temp[camerai]->group.iy /= targets_temp[camerai]->numDots;
						targets_temp[camerai]->group.wx /= targets_temp[camerai]->numDots;
						targets_temp[camerai]->group.wy /= targets_temp[camerai]->numDots;
						targets_temp[camerai]->group.wz /= targets_temp[camerai]->numDots;
						targets_temp[camerai] = targets_temp[camerai]->nxt;
					}
/*
					//find bounding rect of TARGETS  (min enclosing rect that contains all points)
					targets_temp[camerai] = first_target;
					while(targets_temp[camerai])
					{
						targets_temp[camerai]->x1 = targets_temp[camerai]->x2 = targets_temp[camerai]->dots[0].ix;
						targets_temp[camerai]->y1 = targets_temp[camerai]->y2 = targets_temp[camerai]->dots[0].iy;
						for(int i = 0; i<targets_temp[camerai]->numDots; i++)
						{
							if ( targets_temp[camerai]->dots[i].ix < targets_temp[camerai]->x1)
								targets_temp[camerai]->x1 = targets_temp[camerai]->dots[i].ix;
							
							if ( targets_temp[camerai]->dots[i].iy < targets_temp[camerai]->y1)
								targets_temp[camerai]->y1 = targets_temp[camerai]->dots[i].iy;
							
							if ( targets_temp[camerai]->dots[i].ix > targets_temp[camerai]->x2)
								targets_temp[camerai]->x2 = targets_temp[camerai]->dots[i].ix;
							
							if ( targets_temp[camerai]->dots[i].iy > targets_temp[camerai]->y2)
								targets_temp[camerai]->y2 = targets_temp[camerai]->dots[i].iy;
						}
						
						targets_temp[camerai]->x1 = targets_temp[camerai]->x1 - max_dot_distance;
						targets_temp[camerai]->x2 = targets_temp[camerai]->x2 + max_dot_distance;
						targets_temp[camerai]->y1 = targets_temp[camerai]->y1 - max_dot_distance;
						targets_temp[camerai]->y2 = targets_temp[camerai]->y2 + max_dot_distance;
						targets_temp[camerai] = targets_temp[camerai]->nxt;
					}
*/


					//Find interdottal distances
					//This is the unique distance between a set of dots
					//Will be used to match a target to a classifier
					targets_temp[camerai] = first_target;
					while(targets_temp[camerai])
					{
						int count = 0;
						for(int i = 0; i<targets_temp[camerai]->numDots; i++)
						{
							for(int j = i+1; j<targets_temp[camerai]->numDots; j++)
							{
								double dist = sqrt(pow((targets_temp[camerai]->dots[i].wx - targets_temp[camerai]->dots[j].wx),2) + pow( (targets_temp[camerai]->dots[i].wy - targets_temp[camerai]->dots[j].wy),2));
								targets_temp[camerai]->idd[count] = dist;
								count++;
							}
						}
						targets_temp[camerai]->num_idd = count;
						targets_temp[camerai] = targets_temp[camerai]->nxt;
					}

					//Find smallest idd
					//Smallest "inter-dot-distance" used to normalize the distances between the dots
					targets_temp[camerai] = first_target;
					while(targets_temp[camerai])
					{
						targets_temp[camerai]->smallest_idd = targets_temp[camerai]->idd[0];
						for(int i = 0; i<targets_temp[camerai]->num_idd; i++)
						{
							if (targets_temp[camerai]->idd[i]<targets_temp[camerai]->smallest_idd)
							{	
								targets_temp[camerai]->smallest_idd = targets_temp[camerai]->idd[i];
							}
						}
						targets_temp[camerai] = targets_temp[camerai]->nxt;
					}

					//Normalize to smallest idd
					//sort distances in ascending order
					targets_temp[camerai] = first_target;
					while(targets_temp[camerai])
					{
						// use the quick sort function at the top of the program
						q_sort(targets_temp[camerai]->idd, 0, targets_temp[camerai]->num_idd-1);
						//normalize
						for(int i = 0; i<targets_temp[camerai]->num_idd; i++)
						{
							targets_temp[camerai]->idd[i] /= targets_temp[camerai]->smallest_idd;
						}
						targets_temp[camerai] = targets_temp[camerai]->nxt;
					}

					//peform comparison between IDD
					targets_temp[camerai] = first_target;
					while(targets_temp[camerai])	//for each target
					{
						current_classifier = first_classifier;
						targets_temp[camerai]->min_error = -1;
						targets_temp[camerai]->ID = -1;
						while(current_classifier)	//for each classifier (as calculated form the input file "target_classifiers.txt"
						{
							if (current_classifier->numDots == targets_temp[camerai]->numDots)	//if Number of DOTS match
							{
								double error = 0;
								for (int i = 0; i<targets_temp[camerai]->num_idd; i++)	//calculate min squared error
								{
									error += pow(current_classifier->idd[i] - targets_temp[camerai]->idd[i], 2);
								}
								if (error < targets_temp[camerai]->min_error || targets_temp[camerai]->min_error == -1)
								{
									targets_temp[camerai]->min_error = error;
									targets_temp[camerai]->ID = current_classifier->ID;
									targets_temp[camerai]->angleoffset = current_classifier->angleoffset;
									targets_temp[camerai]->centermag = current_classifier->centermag; //new
									targets_temp[camerai]->centeroffset = current_classifier->centeroffset; //new
								}
							}
							current_classifier = current_classifier->nxt;
						}
						targets_temp[camerai] = targets_temp[camerai]->nxt;
					}

					//find closest dot in the target to center of mass
					targets_temp[camerai] = first_target;
					while(targets_temp[camerai])
					{
						double min_dist = -1;
						for(int i = 0; i<targets_temp[camerai]->numDots; i++)
						{
							double temp_dist = sqrt(pow((targets_temp[camerai]->group.wx - targets_temp[camerai]->dots[i].wx),2) + pow( (targets_temp[camerai]->group.wy  - targets_temp[camerai]->dots[i].wy),2));
							if (temp_dist < min_dist || min_dist == -1)
							{
								min_dist = temp_dist;
								targets_temp[camerai]->minDot = targets_temp[camerai]->dots[i];
							}
						}
						targets_temp[camerai] = targets_temp[camerai]->nxt;
					}

					//calculate the angle of the line between the center of mass and the closest dot
					targets_temp[camerai] = first_target;
					while(targets_temp[camerai])
					{
						if (targets_temp[camerai]->minDot.wx < targets_temp[camerai]->group.wx)
						{
							if(targets_temp[camerai]->minDot.wy < targets_temp[camerai]->group.wy)				//quadrant 3				
							{
								targets_temp[camerai]->angle = atan((targets_temp[camerai]->minDot.wy - targets_temp[camerai]->group.wy)/(targets_temp[camerai]->minDot.wx - targets_temp[camerai]->group.wx));
								targets_temp[camerai]->angle = 180 + targets_temp[camerai]->angle*180.0/3.14159;
							}
							else												//quadrant 2
							{
								targets_temp[camerai]->angle = atan((targets_temp[camerai]->minDot.wy - targets_temp[camerai]->group.wy)/(targets_temp[camerai]->minDot.wx - targets_temp[camerai]->group.wx));
								targets_temp[camerai]->angle = 180 + targets_temp[camerai]->angle*180.0/3.14159;
							}
						}
						else
						{
							if(targets_temp[camerai]->minDot.wy < targets_temp[camerai]->group.wy)				//quadrant 4
							{
								targets_temp[camerai]->angle = atan((targets_temp[camerai]->minDot.wy - targets_temp[camerai]->group.wy)/(targets_temp[camerai]->minDot.wx - targets_temp[camerai]->group.wx));
								targets_temp[camerai]->angle = targets_temp[camerai]->angle*180.0/3.14159;
							}
							else												//quandrant 1
							{
								targets_temp[camerai]->angle = atan((targets_temp[camerai]->minDot.wy - targets_temp[camerai]->group.wy)/(targets_temp[camerai]->minDot.wx - targets_temp[camerai]->group.wx));
								targets_temp[camerai]->angle = targets_temp[camerai]->angle*180.0/3.14159;
							}
						}
						/*
						cout<<"("<<targets_temp[camerai]->minDot.wx<<","<<targets_temp[camerai]->minDot.wy<<") ";
						cout<<"("<<targets_temp[camerai]->group.wx<<","<<targets_temp[camerai]->group.wy<<") ";
						cout<<"offset: "<<targets_temp[camerai]->angleoffset;
						cout<<"angle: "<<targets_temp[camerai]->angle<<endl;
						*/
						
						//with the angle above being fixed, subtracting an offset angle as determined by e-puck ID will yield the e-pucks orientation
						targets_temp[camerai]->angle -= targets_temp[camerai]->angleoffset;
						while(targets_temp[camerai]->angle < 0)
						{
							targets_temp[camerai]->angle += 360;
						}
						targets_temp[camerai]->angle = (int) (targets_temp[camerai]->angle) % 360;

						targets_temp[camerai] = targets_temp[camerai]->nxt;
					}

					//fudge the center of mass of the dots to align with the center of the Epucks //new 8/10
					//wx/wy refers to the world coordinates for the center of mass
					//ix/iy refers to the image coordinates for the center of mass >> here was an attempt to shift the image coordinates so that
					//the center of the circle and the tick mark representing direction would at the originate the center of puck, not the center of mass of the LEDs
					//notice here that the ix and iy are not changed, meaning that while the coordinates are corrected, the circle and tick do not show the change, projecting the original center of the LED coordinates
					targets_temp[camerai] = first_target;
					while(targets_temp[camerai])
					{
						double Adjusted = targets_temp[camerai]->angle + targets_temp[camerai]->centeroffset;

						if (Adjusted - 360 <= 90) //Quadrant 1
						{
							//adjusted -= 360;
							Adjusted *= 3.14159/180;
							targets_temp[camerai]->group.wx = targets_temp[camerai]->group.wx + -1*targets_temp[camerai]->centermag*cos(Adjusted);
							targets_temp[camerai]->group.wy = targets_temp[camerai]->group.wy + -1*targets_temp[camerai]->centermag*sin(Adjusted);
							//targets_temp[camerai]->group.ix = targets_temp[camerai]->group.ix + -1*targets_temp[camerai]->centermag*cos(Adjusted);
							//targets_temp[camerai]->group.iy = targets_temp[camerai]->group.iy + -1*targets_temp[camerai]->centermag*sin(Adjusted);
						}
						else if((Adjusted - 360 > 90) && (Blah - 360 <= 180)) //Quadrant 2
						{
							//Adjusted -= 360;
							Blah *= 3.14159/180;
							targets_temp[camerai]->group.wx = targets_temp[camerai]->group.wx + -1*targets_temp[camerai]->centermag*cos(Adjusted);
							targets_temp[camerai]->group.wy = targets_temp[camerai]->group.wy + -1*targets_temp[camerai]->centermag*sin(Adjusted);
							//targets_temp[camerai]->group.ix = targets_temp[camerai]->group.ix + -1*targets_temp[camerai]->centermag*cos(Adjusted);
							//targets_temp[camerai]->group.iy = targets_temp[camerai]->group.iy + -1*targets_temp[camerai]->centermag*sin(Adjusted);
						}
						else if((Adjusted - 360 > 180) && (Blah - 360 <= 270)) //Quadrant 3
						{
							//Adjusted -= 360;
							Blah *= 3.14159/180;
							targets_temp[camerai]->group.wx = targets_temp[camerai]->group.wx + -1*targets_temp[camerai]->centermag*cos(Adjusted);
							targets_temp[camerai]->group.wy = targets_temp[camerai]->group.wy + -1*targets_temp[camerai]->centermag*sin(Adjusted);
							//targets_temp[camerai]->group.ix = targets_temp[camerai]->group.ix + -1*targets_temp[camerai]->centermag*cos(Adjusted);
							//targets_temp[camerai]->group.iy = targets_temp[camerai]->group.iy + -1*targets_temp[camerai]->centermag*sin(Adjusted);
						}
						else //Quadrant 4
						{
							//Adjusted -= 360;
							Blah *= 3.14159/180;
							targets_temp[camerai]->group.wx = targets_temp[camerai]->group.wx + -1*targets_temp[camerai]->centermag*cos(Adjusted);
							targets_temp[camerai]->group.wy = targets_temp[camerai]->group.wy + -1*targets_temp[camerai]->centermag*sin(Adjusted);
							//targets_temp[camerai]->group.ix = targets_temp[camerai]->group.ix + -1*targets_temp[camerai]->centermag*cos(Adjusted);
							//targets_temp[camerai]->group.iy = targets_temp[camerai]->group.iy + -1*targets_temp[camerai]->centermag*sin(Adjusted);
						}					

						targets_temp[camerai] = targets_temp[camerai]->nxt;	
					
					}

					targets_temp[camerai] = first_target;

				}
			}
		} 

		// ALL 4 cameras are processed
		// Time to output resultant captured data


		// total count of patterns from 4 cameras
		int t_count = 0;
		
		//******************************************************//
		//********PHASE 3c: BLIND SPOT REJECTION****************//
		//******************************************************//	
		// used to reject multiple patterns located or partials.

		// process each cameras linkedlist of targets
		for (int camerai = 0; camerai<4; camerai++)
		{
			//initialize head of LL
			target* temp = targets_temp[camerai];
			cvRectangle( thresholdedImageColor[cameraID[camerai]], cvPoint(0,0),cvPoint(fWidth, fHeight), CV_RGB(0,0,255), 1 );

			//temp->group.ix == targets_temp[camerai]->group.wx;
			//temp->group.iy == targets_temp[camerai]->group.wy;

			// process entire LL for this head
			while(temp)
			{
				//remove targets too close to edge of camera frame
				if (temp->group.ix < IGNORE_MARGIN || 
					temp->group.ix >(fWidth-IGNORE_MARGIN) ||
					temp->group.iy < IGNORE_MARGIN ||
					temp->group.iy > fHeight-IGNORE_MARGIN)
				{
						temp->ID = -1;
				}

				if(temp->ID != -1)
				{
					// Check every other target from all other cameras
					for (int camerai2 = 0; camerai2<4; camerai2++)
					{
						target* temp2 = targets_temp[camerai2];

						while(temp2)	// if more targets in linked list
						{
							if(temp2->ID != -1)		// if the ID is a legitmate target (! -1)
							{
								if ((temp2 != temp) || (camerai != camerai2))	//ensure we're not comparing target to itself
								{
									// Calculate distance between patterns
									double temp_dist = sqrt(pow(temp->group.wx - temp2->group.wx,2) + pow( (temp->group.wy  - temp2->group.wy),2));

									// same pattern recognized
									// both cameras saw full images of the targets and their IDs match
									if (temp->ID == temp2->ID)
									{
										//average parameters (x and y coordintes)
										//temp->group.wx = (temp->group.wx + temp2->group.wx) / 2;
										//temp->group.wy = (temp->group.wy + temp2->group.wy) / 2;
										//reset temp2
										temp2->ID = -1;
										
									}
									else if (temp_dist < max_dot_distance*BLIND_SPOT_REJECT_SCALER)
									{
										// different pattern, but within range, so accept pattern with more dots visible
										if (temp->numDots > temp2->numDots)
										{
											temp2->ID = -1;
										}
										else
										{
											temp->ID = -1;
										}
									}
								}
								
							}
							temp2 = temp2->nxt;
						}
						
					}


					if (temp->ID!= -1)
					{
						char coordBuffer[50];
						char IDbuffer[2];
						double marker_length=20, angle_x, angle_y;
						
						//unit conversion;
						temp->angle = temp->angle*PI/180;

						//update total count
						t_count++;				
						//mark targets on screen
						//draw circle on target
						cvCircle( thresholdedImageColor[cameraID[camerai]], cvPoint(temp->group.ix,temp->group.iy) , 15, CV_RGB(255,0,0), 1 );
						//draw tick to indicate orientation, -1 is to flip the tick around, as we are driving the puck "backwards" to 
						//accomodate the position of the color sensor as the "front" of the puck
						angle_x = -1*marker_length*cos(-temp->angle); 
						angle_y = -1*marker_length*sin(-temp->angle);
						
						
						
						//cvLine( thresholdedImageColor[cameraID[camerai]], cvPoint(targets_temp[camerai]->group.wx,targets_temp[camerai]->group.wy), cvPoint(targets_temp[camerai]->group.wx + angle_x ,targets_temp[camerai]->group.wy + angle_y), CV_RGB(255,0,0), 1 );
						cvLine( thresholdedImageColor[cameraID[camerai]], cvPoint(temp->group.ix,temp->group.iy), cvPoint(temp->group.ix + angle_x ,temp->group.iy + angle_y), CV_RGB(255,0,0), 1 );
						
						//print ID and print Coordinates, shift text around when
						//close to the edge of the screen so it won't get clipped
						cvPutText(thresholdedImageColor[cameraID[camerai]], itoa(temp->ID,IDbuffer,10), cvPoint(temp->group.ix + sign(thumbWidth-(temp->group.ix+30))*30,temp->group.iy), &gui_font,  CV_RGB(255,0,0));
						sprintf(coordBuffer,"%5.2f , %5.2f",temp->group.wx, temp->group.wy);
						double text_shift;
						if (temp->group.ix > (thumbWidth-125))
						{
							text_shift = -125;
						}
						else
						{
							text_shift = 0;
						}
						cvPutText(thresholdedImageColor[cameraID[camerai]],coordBuffer,cvPoint(temp->group.ix+text_shift,temp->group.iy+sign(thumbHeight-(temp->group.iy+30))*30),&gui_font_2,CV_RGB(255,0,0));

						//output to console
						//cout<<"Camera "<<cameraID[camerai]<<" Object "<<t_count<<": ("<<temp->ID<<","<<temp->group.wx<<","<<temp->group.wy<<","<<temp->angle<<")"<<endl;
						//cout<<"Camera "<<cameraID[camerai]<<" Object "<<t_count<<": ("<<temp->ID<<","<<temp->group.wx<<","<<temp->group.wy<<","<<temp->angle<<")"<<endl;
						//fprintf(stdout,"%8d  %12.3f  %12.3f  %12.3f\n",temp->ID, temp->group.wx, temp->group.wy, temp->angle);

						//output to filestream
						//fOut<<clock()<<","<<temp->ID<<","<<temp->group.wx<<","<<temp->group.wy<<","<<temp->angle<<endl;

						
						//output to serial port >> this section sends out location data to each robot seen by the vision system
						rawData[0]= 0.0;
						rawData[1]=(float)temp->group.wx;
						rawData[2]=(float)temp->group.wy;
						rawData[3]=(float)normalize_angle(temp->angle);
	

						if (send_counter == SEND_COUNTER_PERIOD)
						{
							bytes_written=packetHandler.assemblePacket(rawData,(UINT16)temp->ID);
							send_counter = 0;
						}
						else{
							send_counter ++;
						}
						//cout<<bytes_written<<endl;
						
/*
						ostringstream oss;
						string serialData;
						oss.str("");
						oss<<temp->ID<<","<<temp->group.wx<<","<<temp->group.wy<<","<<temp->angle<<endl;
//						serialData = oss.str();
//						serialout.Write(serialData.c_str());
*/
					}
				}
				
				temp = temp->nxt;
			}
			
		}

		//Draw the GUI with red circles at targets
		draw_gui_NoCvtColor();

		//Update FPS and target data for GUI
		message1 = "Real-Time Localization Active. Press 'p' to pause, 'c' for command console, and 'Esc' to Quit";
		ostringstream oss;
		oss.str("");
		oss<<t_count<<" targets at "<<getFPS()<<" fps.";
		message2 = oss.str();
		
		//delete linked list of targets
		for(int camerai = 0; camerai<4; camerai++)
		{
			target* temp1 = targets_temp[camerai];
			target* temp_old;		
			while(temp1)
			{
				temp_old = temp1;
				temp1 = temp1->nxt;
				delete temp_old;
			}
		}

		//CHECK KEYSTROKES
		if(process_keys()==1)
		{
			break;
		}

	}

	// Get here when user presses escape, otherwise this loops back up to main program loop


	//END OF PROGRAM, FREE UP RESOURCES\
	//Close serial port

	COM.Flush();
	COM.Close();

	//REALEASE CAMERAS
	for(int camerai = 0; camerai<4; camerai++)
	{
		VI->stopDevice(cameraID[camerai]);
	}

	//RELEASE FILESTREAM 
	fOut.close();
	
	//RELEASE IMAGES
	for(int camerai = 0; camerai<4; camerai++)
	{
		cvReleaseImage(&greyImage[camerai]);
		cvReleaseImage(&thresholdedImageColor[camerai]);
		cvReleaseImage(&thresholdedImage[camerai]);
	}
	cvReleaseImage(&image);
	cvReleaseImage(&grey);
	cvReleaseImage(&thresholded_image);
	cvReleaseMemStorage( &storage);
    cvDestroyWindow(mainWindow.c_str());
	cvDestroyWindow(vidWindow.c_str());
	delete(VI);
	exit(333);

    return 0;
}

#ifdef _EiC
main(1,"lkdemo.c");
#endif